﻿//////////////////////////////////////////////
// RasterStateManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class RasterState ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// nkCommon
#include <nilkinsCommon/Patterns/SingletonClass.h>

// nkMemory
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <mutex>
#include <string>
#include <unordered_map>

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT RasterStateManager : public nkCommon::SingletonClass<RasterStateManager>
	{
		friend class nkCommon::SingletonClass<RasterStateManager> ;

		private :

			// Attributs
			// Liste
			std::unordered_map<std::string, RasterState*> _memory ;

			// Default state
			RasterState* _defaultState ;

			// Mutex
			std::mutex _mutex ;

		private :

			// Functions
			// Constructeur
			RasterStateManager () ;
		
		public :

			// Destructeur
			~RasterStateManager () ;
		
			// Manipulation mémoire
			RasterState* createOrRetrieve (const nkMemory::StringView& name) ;
			RasterState* get (const nkMemory::StringView& name) ;
			RasterState* getByIndex (unsigned int index) ;
			void rename (const nkMemory::StringView& oldName, const nkMemory::StringView& newName) ;
			void erase (const nkMemory::StringView& name) ;

			// Getters
			RasterState* getDefaultState () const ;
	} ;
}